@@ -229,6 +229,10 @@ if [ "${DO_SERVICE}" == "1" ]; then
 					fi
 				done
 			fi
+			LINK=`find "$PF\\Microsoft Visual Studio" -name "link.exe" | grep -i "Hostx64/x64/link.exe" | sort -n | head -n 1`
+			if [ ! -z "${LINK}" ]; then
+				break 2
+			fi
 		done
 	done
 	#MC="C:\\Program Files\\Windows Kits\\8.1\\bin\\x64\\mc.exe"
@@ -498,6 +502,11 @@ if [ "${BUNDLE_DESKTOPLOGON}" == "1" ]; then
 	done
 fi
 
+if [ "${DO_CUDA}" == "1" ]; then
+	#pycuda wants a CUDA_PATH with "/bin" in it:
+	mkdir "${DIST}/bin"
+fi
+
 if [ "${DO_VERPATCH}" == "1" ]; then
 	for exe in `ls dist/*exe | grep -v Plink.exe`; do
 		tool_name=`echo $exe | sed 's+dist/++g;s+Xpra_++g;s+Xpra-++g;s+_+ +g;s+-+ +g;s+\.exe++g'`
