@@ -1,15 +1,15 @@
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
-# Copyright (C) 2012-2019 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2012-2022 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import os
+import re
 from time import monotonic
-from collections import namedtuple
 from ctypes import (
     create_string_buffer, create_unicode_buffer,
-    sizeof, byref, addressof, c_int,
+    sizeof, byref, addressof, c_ulong,
     )
 from ctypes.wintypes import RECT, POINT, BYTE, MAX_PATH
 
@@ -220,13 +220,12 @@ def init_capture(w, h, pixel_depth=32):
 
 class SeamlessRootWindowModel(RootWindowModel):
 
-    def __init__(self, root, capture):
-        super().__init__(root, capture)
+    def __init__(self, root, capture, title, geometry):
+        super().__init__(root, capture, title, geometry)
         log("SeamlessRootWindowModel(%s, %s) SEAMLESS=%s", root, capture, SEAMLESS)
         self.property_names.append("shape")
         self.dynamic_property_names.append("shape")
         self.rectangles = self.get_shape_rectangles(logit=True)
-        self.shape_notify = []
 
     def refresh_shape(self):
         rectangles = self.get_shape_rectangles()
@@ -234,21 +233,7 @@ class SeamlessRootWindowModel(RootWindowModel):
             return  #unchanged
         self.rectangles = rectangles
         shapelog("refresh_shape() sending notify for updated rectangles: %s", rectangles)
-        #notify listeners:
-        PSpec = namedtuple("PSpec", "name")
-        pspec = PSpec(name="shape")
-        for cb, args in self.shape_notify:
-            shapelog("refresh_shape() notifying: %s", cb)
-            try:
-                cb(self, pspec, *args)
-            except Exception:
-                shapelog.error("error in shape notify callback %s", cb, exc_info=True)
-
-    def connect(self, signal, cb, *args):
-        if signal=="notify::shape":
-            self.shape_notify.append((cb, args))
-        else:
-            super().connect(signal, cb, *args)
+        self.notify("shape")
 
     def get_shape_rectangles(self, logit=False):
         #get the list of windows
@@ -264,7 +249,7 @@ class SeamlessRootWindowModel(RootWindowModel):
             if not IsWindowVisible(hwnd):
                 l("skipped invisible window %#x", hwnd)
                 return True
-            pid = c_int()
+            pid = c_ulong()
             thread_id = GetWindowThreadProcessId(hwnd, byref(pid))
             if pid==ourpid:
                 l("skipped our own window %#x", hwnd)
@@ -327,7 +312,7 @@ class SeamlessRootWindowModel(RootWindowModel):
                 h = bottom
             rectangles.append((left, top, w, h))
             return True
-        EnumWindows(EnumWindowsProc(enum_windows_cb))
+        EnumWindows(EnumWindowsProc(enum_windows_cb), 0)
         l("get_shape_rectangles()=%s", rectangles)
         return sorted(rectangles)
 
@@ -339,6 +324,22 @@ class SeamlessRootWindowModel(RootWindowModel):
         return super().get_property(prop)
 
 
+class Win32ShadowModel(RootWindowModel):
+    __slots__ = ("hwnd", "iconic")
+    def __init__(self, root_window, capture=None, title="", geometry=None):
+        super().__init__(root_window, capture, title, geometry)
+        self.hwnd = 0
+        self.iconic = geometry[2]==-32000 and geometry[3]==-32000
+        self.property_names.append("hwnd")
+        self.dynamic_property_names.append("size-hints")
+
+    def get_id(self):
+        return self.hwnd
+
+    def __repr__(self):
+        return "Win32ShadowModel(%s : %24s : %s)" % (self.capture, self.geometry, self.hwnd)
+
+
 class ShadowServer(GTKShadowServerBase):
 
     def __init__(self):
@@ -413,6 +414,74 @@ class ShadowServer(GTKShadowServerBase):
             return SeamlessRootWindowModel
         return RootWindowModel
 
+
+    def makeDynamicWindowModels(self):
+        assert self.window_matches
+        ourpid = os.getpid()
+        taskbar = FindWindowA("Shell_TrayWnd", None)
+        windows = {}
+        def enum_windows_cb(hwnd, lparam):
+            if not IsWindowVisible(hwnd):
+                log("window %#x is not visible", hwnd)
+                return True
+            pid = c_ulong()
+            thread_id = GetWindowThreadProcessId(hwnd, byref(pid))
+            if pid==ourpid:
+                log("skipped our own window %#x, thread id=%#x", hwnd, thread_id)
+                return True
+            rect = RECT()
+            if GetWindowRect(hwnd, byref(rect))==0:
+                log("GetWindowRect failure")
+                return True
+            if hwnd==taskbar:
+                log("skipped taskbar")
+                return True
+            #skipping IsWindowEnabled check
+            length = GetWindowTextLengthW(hwnd)
+            buf = create_unicode_buffer(length+1)
+            if GetWindowTextW(hwnd, buf, length+1)>0:
+                window_title = buf.value
+            else:
+                window_title = ''
+            left, top, right, bottom = rect.left, rect.top, rect.right, rect.bottom
+            w = right-left
+            h = bottom-top
+            if left<=-32000 or top<=-32000:
+                log("%r is not visible: %s", window_title, (left, top, w, h))
+            if w<=0 and h<=0:
+                log("skipped invalid window size: %ix%i", w, h)
+                return True
+            windows[hwnd] = (window_title, (left, top, w, h))
+            return True
+        EnumWindows(EnumWindowsProc(enum_windows_cb), 0)
+        log("makeDynamicWindowModels() windows=%s", windows)
+        models = []
+        def add_model(hwnd, title, geometry):
+            model = Win32ShadowModel(self.root, self.capture, title=title, geometry=geometry)
+            model.hwnd = hwnd
+            models.append(model)
+        for m in self.window_matches:
+            window = None
+            try:
+                if m.startswith("0x"):
+                    hwnd = int(m, 16)
+                else:
+                    hwnd = int(m)
+                if hwnd:
+                    window = windows.pop(hwnd, None)
+                    if window:
+                        add_model(hwnd, *window)
+            except ValueError:
+                namere = re.compile(m, re.IGNORECASE)
+                for hwnd, window in tuple(windows.items()):
+                    title, geometry = window
+                    if namere.match(title):
+                        add_model(hwnd, title, geometry)
+                        windows.pop(hwnd)
+        log("makeDynamicWindowModels()=%s", models)
+        return models
+
+
     def get_shadow_monitors(self):
         #convert to the format expected by GTKShadowServerBase:
         monitors = []
@@ -521,9 +590,9 @@ class ShadowServer(GTKShadowServerBase):
         pointer = self._process_mouse_common(proto, wid, pointer)
         if pointer:
             self.get_server_source(proto).user_event()
-            self.button_action(pointer, button, pressed, -1, *args)
+            self.button_action(wid, pointer, button, pressed, -1, *args)
 
-    def button_action(self, pointer, button, pressed, deviceid=-1, *args):
+    def button_action(self, wid, pointer, button, pressed, deviceid=-1, *args):
         event = BUTTON_EVENTS.get((button, pressed))
         if event is None:
             log.warn("no matching event found for button=%s, pressed=%s", button, pressed)
