@@ -7,6 +7,7 @@
 import os.path
 import sys
 import site
+import tempfile
 
 
 def do_get_desktop_background_paths():
@@ -61,17 +62,11 @@ def do_get_icon_dir():
     from xpra.platform.paths import get_app_dir
     return os.path.join(get_app_dir(), "icons")
 
-def do_get_libexec_dir():
-    from xpra.os_util import is_Fedora, is_CentOS, is_RedHat
-    if is_Fedora() or is_CentOS() or is_RedHat():
-        return "/usr/libexec/"
-    return "/usr/lib"
-
 def do_get_mmap_dir():
-    return _get_xpra_runtime_dir() or os.getenv("TMPDIR", "/tmp")
+    return _get_xpra_runtime_dir() or tempfile.gettempdir()
 
 def do_get_xpra_tmp_dir():
-    xrd = _get_xpra_runtime_dir()
+    xrd = os.environ.get("XPRA_SESSION_DIR", _get_xpra_runtime_dir())
     return os.path.join(xrd, "tmp")
 
 
@@ -115,12 +110,15 @@ def do_get_user_conf_dirs(uid):
 
 def get_runtime_dir():
     runtime_dir = os.environ.get("XDG_RUNTIME_DIR")
-    if runtime_dir or sys.platform.startswith("linux"):
-        return "$XDG_RUNTIME_DIR"
-    for d in ("/run/user", "/var/run/user"):
-        if os.path.exists(d) and os.path.isdir(d):
-            runtime_dir = d+"/$UID"
-            break
+    if runtime_dir:
+        return runtime_dir
+    if sys.platform.startswith("linux"):
+        for d in ("/run/user", "/var/run/user"):
+            if os.path.exists(d) and os.path.isdir(d):
+                runtime_dir = d+"/$UID"
+                break
+        if not runtime_dir:
+            return "$XDG_RUNTIME_DIR"
     return runtime_dir
 
 def _get_xpra_runtime_dir():
@@ -160,7 +158,7 @@ def do_get_default_log_dirs():
     v = _get_xpra_runtime_dir()
     if v:
         log_dirs.append(v)
-    log_dirs.append("/tmp")
+    log_dirs.append(tempfile.gettempdir())
     return log_dirs
 
 def do_get_sound_command():
